@@ -1,24 +1,20 @@
 """ A list of helpful decorators for use throughout ParmEd """
+from functools import wraps
+import warnings
 
-__all__ = ['needs_openmm']
-
-from parmed.utils.six import wraps
 try:
-    import simtk.openmm as mm
-    import simtk.openmm.app as app
+    import openmm as mm
     HAS_OPENMM = True
 except ImportError:
     HAS_OPENMM = False
-else:
-    del mm, app
-import warnings
+
+__all__ = ['needs_openmm']
 
 def needs_openmm(fcn):
-    global HAS_OPENMM
     @wraps(fcn)
     def new_fcn(*args, **kwargs):
         if not HAS_OPENMM:
-            raise ImportError('Could not find or import OpenMM')
+            raise ImportError('Could not find or import OpenMM version 7.0+')
         return fcn(*args, **kwargs)
 
     return new_fcn
@@ -26,7 +22,8 @@ def needs_openmm(fcn):
 def deprecated(fcn):
     @wraps(fcn)
     def new_fcn(*args, **kwargs):
-        warnings.warn('%s is deprecated and will be removed in the future' %
-                      fcn.__name__, DeprecationWarning)
+        warnings.warn(
+            f'{fcn.__name__} is deprecated and will be removed in the future', DeprecationWarning
+        )
         return fcn(*args, **kwargs)
     return new_fcn
