@@ -1,20 +1,19 @@
 """
 Tests for the parmed/charmm subpackage
 """
-from __future__ import division, print_function
-
-from collections import OrderedDict, defaultdict
+from collections import defaultdict
+from io import StringIO
 import copy
 import numpy as np
+import pytest
 import os
 import parmed as pmd
 from parmed.utils.io import genopen
-from parmed.utils.six import iteritems, string_types
-from parmed.utils.six.moves import StringIO
 from parmed.charmm import charmmcrds, parameters, psf
 from parmed.charmm._charmmfile import CharmmFile, CharmmStreamFile
 from parmed import exceptions, topologyobjects as to, load_file, ParameterSet
-from parmed.topologyobjects import BondType, AngleType, DihedralType, DihedralTypeList
+from parmed.topologyobjects import Atom, BondType, AngleType, DihedralType, DihedralTypeList, DrudeAtom, ExtraPoint
+from parmed.periodic_table import Mass
 import parmed.unit as u
 import random
 import unittest
@@ -25,10 +24,19 @@ class TestCharmmBase(FileIOTestCase):
 
     def __init__(self, *args, **kwargs):
         super().__init__(*args, **kwargs)
+        warnings.simplefilter("ignore", category=exceptions.ParameterWarning)
         self.param22 = parameters.CharmmParameterSet(
             get_fn('top_all22_prot.inp'), get_fn('par_all22_prot.inp')
         )
 
+    def setUp(self):
+        warnings.simplefilter("ignore", category=exceptions.ParameterWarning)
+        super().setUp()
+
+    def tearDown(self):
+        warnings.simplefilter("always", category=exceptions.ParameterWarning)
+        return super().tearDown()
+
 class TestCharmmCoords(TestCharmmBase):
     """ Test CHARMM coordinate file parsers """
 
@@ -257,6 +265,14 @@ class TestCharmmPsf(TestCharmmBase):
         for res in (firstres * 3).residues:
             self.assertEqual(res.segid, firstres.residues[0].segid)
 
+    def test_psf_atomic_number_assignment(self):
+        """ Checks that atomic number is assigned if just PSF is read """
+        psf1 = psf.CharmmPsfFile(get_fn('ala_ala_ala.psf'))
+        psf2 = psf.CharmmPsfFile(get_fn('ala_ala_ala.psf'))
+        psf2.load_parameters(parmset=self.param22)
+        for a1, a2 in zip(psf1.atoms, psf2.atoms):
+            self.assertEqual(a1.atomic_number, a2.atomic_number)
+
     def test_xplor_psf(self):
         """ Test Xplor-format CHARMM PSF file parsing """
         # Atom types are strings, not integers like in charmm
@@ -586,46 +602,40 @@ class TestCharmmParameters(TestCharmmBase):
         params = parameters.CharmmParameterSet(
                 get_fn('parm14sb_all.prm'),
         )
-        for i, tortype in iteritems(params.dihedral_types):
+        for i, tortype in params.dihedral_types.items():
             for typ in tortype:
                 self.assertAlmostEqual(typ.scee, 1.2)
         params = parameters.CharmmParameterSet(
                 get_fn('parm14sb_all_2.prm')
         )
-        for i, tortype in iteritems(params.dihedral_types):
+        for i, tortype in params.dihedral_types.items():
             for typ in tortype:
                 self.assertAlmostEqual(typ.scee, 1.2)
         # Now test that adding to the parameter set with a DIFFERENT 1-4 scaling
         # factor is caught
-        self.assertRaises(exceptions.CharmmError, lambda:
-                params.read_parameter_file(get_fn('par_all36_prot.prm'))
-        )
-        self.assertRaises(exceptions.CharmmError, lambda:
-                parameters.CharmmParameterSet(get_fn('parm14sb_all.prm'),
-                                              get_fn('dummy_charmm.str'))
-        )
+        with self.assertRaises(exceptions.CharmmError):
+            params.read_parameter_file(get_fn('par_all36_prot.prm'))
+        with self.assertRaises(exceptions.CharmmError):
+            parameters.CharmmParameterSet(get_fn('parm14sb_all.prm'), get_fn('dummy_charmm.str'))
 
     def test_geometric(self):
         """ Test reading CHARMM parameter file with geometric comb. rule """
-        opls = parameters.CharmmParameterSet(get_fn('top_opls_aa.inp'),
-                                             get_fn('par_opls_aa.inp'))
+        opls = parameters.CharmmParameterSet(get_fn('top_opls_aa.inp'), get_fn('par_opls_aa.inp'))
         self.assertEqual(opls.combining_rule, 'geometric')
         # Now test error handling corresponding to illegal mixing of
         # incompatible parameter files.
         non_opls = parameters.CharmmParameterSet(get_fn('par_all36_prot.prm'))
         self.assertEqual(non_opls.combining_rule, 'lorentz')
         non_opls.read_topology_file(get_fn('top_opls_aa.inp'))
-        self.assertRaises(exceptions.CharmmError, lambda:
-                non_opls.read_parameter_file(get_fn('par_geometric_combining.inp'))
-        )
-        self.assertRaises(exceptions.CharmmError, lambda:
-                non_opls.read_parameter_file(get_fn('par_opls_aa.inp'))
-        )
-        for _, dt in iteritems(opls.dihedral_types):
-            for t in dt: t.scee = t.scnb = 1.0
-        self.assertRaises(exceptions.CharmmError, lambda:
-                opls.read_parameter_file(get_fn('par_all36_prot.prm'))
-        )
+        with self.assertRaises(exceptions.CharmmError):
+            non_opls.read_parameter_file(get_fn('par_geometric_combining.inp'))
+        with self.assertRaises(exceptions.CharmmError):
+            non_opls.read_parameter_file(get_fn('par_opls_aa.inp'))
+        for _, dt in opls.dihedral_types.items():
+            for t in dt:
+                t.scee = t.scnb = 1.0
+        with self.assertRaises(exceptions.CharmmError):
+            opls.read_parameter_file(get_fn('par_all36_prot.prm'))
 
     def test_single_parameterset(self):
         """ Test reading a single parameter set """
@@ -642,8 +652,7 @@ class TestCharmmParameters(TestCharmmBase):
         )
         self._check_single_paramset(
             parameters.CharmmParameterSet.load_set(
-                tfile=get_fn('top_all22_prot.inp'),
-                pfile=get_fn('par_all22_prot.inp'),
+                tfile=get_fn('top_all22_prot.inp'), pfile=get_fn('par_all22_prot.inp')
             )
         )
 
@@ -667,13 +676,13 @@ class TestCharmmParameters(TestCharmmBase):
         # Look at the parsed residue templates and make sure they line up
         self.assertEqual(len(params.residues), 32)
         self.assertEqual(set(params.residues.keys()),
-                set(['ALA', 'ARG', 'ASN', 'ASP', 'CYS', 'GLN', 'GLU', 'GLY',
-                     'HSD', 'HSE', 'HSP', 'ILE', 'LEU', 'LYS', 'MET', 'PHE',
-                     'PRO', 'SER', 'THR', 'TRP', 'TYR', 'VAL', 'ALAD', 'TIP3',
-                     'TP3M', 'SOD', 'MG', 'POT', 'CES', 'CAL', 'CLA', 'ZN2'])
+            set(['ALA', 'ARG', 'ASN', 'ASP', 'CYS', 'GLN', 'GLU', 'GLY',
+                 'HSD', 'HSE', 'HSP', 'ILE', 'LEU', 'LYS', 'MET', 'PHE',
+                 'PRO', 'SER', 'THR', 'TRP', 'TYR', 'VAL', 'ALAD', 'TIP3',
+                 'TP3M', 'SOD', 'MG', 'POT', 'CES', 'CAL', 'CLA', 'ZN2'])
         )
         self.assertEqual(len(params.patches), 22)
-        for resname, res in iteritems(params.residues):
+        for resname, res in params.residues.items():
             if resname in ('TIP3', 'TP3M', 'SOD', 'MG', 'CLA', 'POT', 'CES',
                     'CAL', 'ZN2', 'ALAD'):
                 self.assertIs(res.first_patch, None)
@@ -753,11 +762,10 @@ class TestCharmmParameters(TestCharmmBase):
     def test_write_params(self):
         """ Tests writing CHARMM RTF/PAR/STR files from parameter sets """
         params = parameters.CharmmParameterSet(get_fn('top_all22_prot.inp'), get_fn('par_all22_prot.inp'))
-        params.write(top=self.get_fn('test.rtf', written=True),
-                     par=self.get_fn('test.par', written=True))
-        params.write(str=self.get_fn('test.str', written=True))
+        params.write(top=self.get_fn('test.rtf', written=True), par=self.get_fn('test.par', written=True))
+        params.write(stream=self.get_fn('test.str', written=True))
         # Check bad options
-        self.assertRaises(ValueError, lambda: params.write())
+        self.assertRaises(ValueError, params.write)
 
         params2 = parameters.CharmmParameterSet(
             self.get_fn('test.rtf', written=True), self.get_fn('test.par', written=True)
@@ -866,24 +874,21 @@ class TestCharmmParameters(TestCharmmBase):
 
         # Load a parameter set with NoUreyBradley to make sure it's retained as
         # a singleton. Also build a list of atom type tuples
-        for i, (typstr, typ) in enumerate(iteritems(params1.atom_types)):
+        for i, (typstr, typ) in enumerate(params1.atom_types.items()):
             params1.atom_types_tuple[(typstr, i+1)] = typ
             params1.atom_types_int[i+1] = typ
         for key in params1.angle_types:
             params1.urey_bradley_types[key] = to.NoUreyBradley
         chparams = parameters.CharmmParameterSet.from_parameterset(params1)
-        for _, item in iteritems(chparams.urey_bradley_types):
+        for _, item in chparams.urey_bradley_types.items():
             self.assertIs(item, to.NoUreyBradley)
-        for (typstr1, typ1), (typstr2, typ2) in zip(iteritems(params1.atom_types),
-                                                    iteritems(params1.atom_types)):
+        for (typstr1, typ1), (typstr2, typ2) in zip(params1.atom_types.items(), params1.atom_types.items()):
             self.assertEqual(typstr1, typstr2)
             self.assertEqual(typ1, typ2)
-        for (typstr1, typ1), (typstr2, typ2) in zip(iteritems(params1.atom_types_int),
-                                                    iteritems(params1.atom_types_int)):
+        for (typstr1, typ1), (typstr2, typ2) in zip(params1.atom_types_int.items(), params1.atom_types_int.items()):
             self.assertEqual(typstr1, typstr2)
             self.assertEqual(typ1, typ2)
-        for (typstr1, typ1), (typstr2, typ2) in zip(iteritems(params1.atom_types_tuple),
-                                                    iteritems(params1.atom_types_tuple)):
+        for (typstr1, typ1), (typstr2, typ2) in zip(params1.atom_types_tuple.items(), params1.atom_types_tuple.items()):
             self.assertEqual(typstr1, typstr2)
             self.assertEqual(typ1, typ2)
 
@@ -900,8 +905,7 @@ class TestCharmmParameters(TestCharmmBase):
         )
         gmx.parameterset.nbfix_types[('X', 'Y')] = (2.0, 3.0)
         from_gmx2 = parameters.CharmmParameterSet.from_parameterset(gmx.parameterset)
-        for (key1, typ1), (key2, typ2) in zip(iteritems(from_gmx.cmap_types),
-                                              iteritems(from_gmx2.cmap_types)):
+        for (key1, typ1), (key2, typ2) in zip(from_gmx.cmap_types.items(), from_gmx2.cmap_types.items()):
             self.assertEqual(key1, key2)
             self.assertEqual(typ1, typ2)
         self.assertEqual(len(from_gmx2.nbfix_types), 1)
@@ -926,7 +930,7 @@ class TestCharmmParameters(TestCharmmBase):
 
 
     def _check_uppercase_types(self, params):
-        for aname, atom_type in iteritems(params.atom_types):
+        for aname, atom_type in params.atom_types.items():
             self.assertEqual(aname, aname.upper())
             self.assertEqual(atom_type.name, atom_type.name.upper())
         for key in params.bond_types:
@@ -946,13 +950,13 @@ class TestCharmmParameters(TestCharmmBase):
         def get_typeset(set1, set2):
             ids1 = set()
             ids2 = set()
-            for _, item in iteritems(set1):
+            for _, item in set1.items():
                 ids1.add(id(item))
-            for _, item in iteritems(set2):
+            for _, item in set2.items():
                 ids2.add(id(item))
             return ids1, ids2
         def typenames(key):
-            if isinstance(key, string_types):
+            if isinstance(key, str):
                 return parameters._typeconv(key)
             return tuple(typenames(k) for k in key)
         # Bonds
@@ -962,7 +966,7 @@ class TestCharmmParameters(TestCharmmBase):
             self.assertFalse(b1 & b2)
         else:
             self.assertEqual(b1, b2)
-        for key, item2 in iteritems(set2.bond_types):
+        for key, item2 in set2.bond_types.items():
             self.assertEqual(set1.bond_types[typenames(key)], item2)
         # Angles
         a1, a2 = get_typeset(set1.angle_types, set2.angle_types)
@@ -971,7 +975,7 @@ class TestCharmmParameters(TestCharmmBase):
             self.assertFalse(a1 & a2)
         else:
             self.assertEqual(a1, a2)
-        for key, item2 in iteritems(set2.angle_types):
+        for key, item2 in set2.angle_types.items():
             self.assertEqual(set1.angle_types[typenames(key)], item2)
         # Dihedrals
         d1, d2 = get_typeset(set1.dihedral_types, set2.dihedral_types)
@@ -980,7 +984,7 @@ class TestCharmmParameters(TestCharmmBase):
             self.assertFalse(d1 & d2)
         else:
             self.assertEqual(d1, d2)
-        for key, item2 in iteritems(set2.dihedral_types):
+        for key, item2 in set2.dihedral_types.items():
             self.assertEqual(set1.dihedral_types[typenames(key)], item2)
         # Impropers
         d1, d2 = get_typeset(set1.improper_types, set2.improper_types)
@@ -989,7 +993,7 @@ class TestCharmmParameters(TestCharmmBase):
             self.assertFalse(d1 & d2)
         else:
             self.assertEqual(d1, d2)
-        for key, item2 in iteritems(set2.improper_types):
+        for key, item2 in set2.improper_types.items():
             self.assertEqual(set1.improper_types[typenames(key)], item2)
         # Periodic impropers
         d1, d2 = get_typeset(set1.improper_periodic_types, set2.improper_periodic_types)
@@ -998,7 +1002,7 @@ class TestCharmmParameters(TestCharmmBase):
             self.assertFalse(d1 & d2)
         else:
             self.assertEqual(d1, d2)
-        for key, item2 in iteritems(set2.improper_periodic_types):
+        for key, item2 in set2.improper_periodic_types.items():
             self.assertEqual(set1.improper_periodic_types[typenames(key)], item2)
         # CMAPs
         d1, d2 = get_typeset(set1.cmap_types, set2.cmap_types)
@@ -1007,7 +1011,7 @@ class TestCharmmParameters(TestCharmmBase):
             self.assertFalse(d1 & d2)
         else:
             self.assertEqual(d1, d2)
-        for key, item2 in iteritems(set2.cmap_types):
+        for key, item2 in set2.cmap_types.items():
             self.assertEqual(len(key), 8)
             self.assertEqual(set1.cmap_types[typenames(key)], item2)
         # Atom types
@@ -1036,12 +1040,10 @@ class TestFileWriting(TestCharmmBase):
 
     def test_charmm_file(self):
         """ Test the CharmmFile API and error handling """
-        self.assertRaises(ValueError, lambda:
-                CharmmFile(get_fn('trx.prmtop'), 'x')
-        )
-        self.assertRaises(IOError, lambda:
-                CharmmFile(get_fn('file_does_not_exist'), 'r')
-        )
+        with self.assertRaises(ValueError):
+            CharmmFile(get_fn('trx.prmtop'), 'x')
+        with self.assertRaises(IOError):
+            CharmmFile(get_fn('file_does_not_exist'), 'r')
         with CharmmFile(self.get_fn('newfile.chm', written=True), 'w') as f:
             f.write('abc123\ndef456\nghi789!comment...\n')
         with CharmmFile(self.get_fn('newfile.chm', written=True), 'r') as f:
@@ -1072,10 +1074,8 @@ class TestFileWriting(TestCharmmBase):
                 lines.append(line)
                 comments.append(f.comment)
                 line = f.readline()
-            self.assertEqual(lines, ['First line \n', 'Second line \n',
-                                     'Third line \n', 'Fourth line \n'])
-            self.assertEqual(comments, ['! first comment', '! second comment',
-                                        '! third comment', '! fourth comment'])
+            self.assertEqual(lines, ['First line \n', 'Second line \n', 'Third line \n', 'Fourth line \n'])
+            self.assertEqual(comments, ['! first comment', '! second comment', '! third comment', '! fourth comment'])
 
     def test_charmm_stream_file(self):
         """ Test the CharmmStreamFile API """
@@ -1102,6 +1102,33 @@ class TestFileWriting(TestCharmmBase):
         cpsf.write_psf(fn)
         cpsf2 = psf.CharmmPsfFile(fn)
 
+    def _run_lp_test(self, filename: str, n_lps: int):
+        """ Test writing PSF files with 4-particle lone pairs """
+        cpsf = psf.CharmmPsfFile(get_fn(filename))
+        self.assertEqual(sum(isinstance(atom, ExtraPoint) for atom in cpsf.atoms), n_lps)
+
+        fn = self.get_fn("test.psf", written=True)
+        cpsf.write_psf(fn)
+        cpsf2 = psf.CharmmPsfFile(fn)
+
+        self.assertEqual(sum(isinstance(atom, ExtraPoint) for atom in cpsf2.atoms), n_lps)
+        for atom1, atom2 in zip(cpsf.atoms, cpsf2.atoms):
+            if isinstance(atom1, DrudeAtom):
+                self.assertIsInstance(atom2, DrudeAtom)
+                if atom1.anisotropy is None:
+                    self.assertIsNone(atom2.anisotropy)
+                else:
+                    self.assertGreater(len(atom1.anisotropy.params), 0)
+                    self.assertEqual(atom1.anisotropy.params.keys(), atom2.anisotropy.params.keys())
+                    for key in atom1.anisotropy.params:
+                        self.assertAlmostEqual(atom1.anisotropy.params[key], atom2.anisotropy.params[key], delta=1e-4)
+
+    def test_write_psf_with_lp_1(self):
+        self._run_lp_test("cyt-gua-cyt.psf", 23)
+
+    def test_write_psf_with_lp_2(self):
+        self._run_lp_test("ala3_solv_drude.psf", 909)
+
     def test_eliminate_duplicate_dihedrals(self):
         """ Test that duplicate torsions are eliminated in PSF writes """
         def count_torsions(parm):
@@ -1132,13 +1159,11 @@ class TestFileWriting(TestCharmmBase):
         for attr in dir(cpsf):
             if attr.startswith('_'): continue
             # Skip descriptors
-            if attr in ('topology', 'positions', 'box_vectors',
-                        'velocities', 'name', 'view'):
+            if attr in ('topology', 'positions', 'box_vectors', 'velocities', 'name', 'view', 'rdkit_mol'):
                 continue
             if callable(getattr(cpsf, attr)): continue
             if hasattr(getattr(cpsf, attr), '__len__'):
-                self.assertEqual(len(getattr(cpsf, attr)),
-                                 len(getattr(cpsf2, attr)))
+                self.assertEqual(len(getattr(cpsf, attr)), len(getattr(cpsf2, attr)))
             else:
                 self.assertEqual(getattr(cpsf, attr), getattr(cpsf2, attr))
         f = open(self.get_fn('dhfr_cmap_pbc.psf', written=True), 'r')
@@ -1160,8 +1185,7 @@ class TestFileWriting(TestCharmmBase):
         cpsf2 = psf.CharmmPsfFile(self.get_fn('dhfr_cmap_pbc.psf', written=True))
         for attr in dir(cpsf):
             if attr.startswith('_'): continue
-            if attr in ('topology', 'positions', 'box_vectors',
-                        'velocities', 'name', 'view'):
+            if attr in ('topology', 'positions', 'box_vectors', 'velocities', 'name', 'view', 'rdkit_mol'):
                 continue
             if callable(getattr(cpsf, attr)): continue
             if hasattr(getattr(cpsf, attr), '__len__'):
@@ -1187,3 +1211,66 @@ class TestFileWriting(TestCharmmBase):
         parm.save(fn, overwrite=True)
         cpsf = pmd.load_file(fn)
         self.assertIn('XPLOR', cpsf.flags)
+
+class TestCharmmDrudeSystems(TestCharmmBase):
+    """ Tests processing of DRUDE systems """
+
+    def test_drude_mass(self):
+        drude_psf = psf.CharmmPsfFile(get_fn("ala3_solv_drude.psf"))
+        params = parameters.CharmmParameterSet(get_fn("toppar_drude_master_protein_2013e.str"))
+        drude_psf.load_parameters(params)
+        # Check this against a known-correct implementation in OpenM
+        self.assertEqual(sum([isinstance(a, DrudeAtom) for a in drude_psf.atoms]), 957)
+        for atom in drude_psf.atoms:
+            if isinstance(atom, DrudeAtom):
+                self.assertIsInstance(atom.parent, Atom)
+        for atom in drude_psf.atoms:
+            if isinstance(atom, DrudeAtom):
+                self.assertAlmostEqual(atom.mass + atom.parent.mass, Mass[atom.parent.element_name], delta=0.01)
+
+        self.assertEqual(sum(atom.anisotropy is not None for atom in drude_psf.atoms if isinstance(atom, DrudeAtom)), 4)
+        self.assertTrue(any(atom.anisotropy is None for atom in drude_psf.atoms if isinstance(atom, DrudeAtom)))
+
+
+    def test_drude_parsing_na(self):
+        """ Test parsing a trinucleotide with Drude particles and lone pairs """
+        system = psf.CharmmPsfFile(get_fn("cyt-gua-cyt.psf"))
+        self.assertEqual(len(system.atoms), 176)
+
+        # Check that we have the right number of DRUDE particles
+        self.assertEqual(sum(isinstance(atom, DrudeAtom) for atom in system.atoms), 59)
+        self.assertTrue(all(atom.type == "DRUD" for atom in system.atoms if isinstance(atom, DrudeAtom)))
+
+        # Check that we have the right number of lone pairs
+        self.assertEqual(sum(isinstance(atom, ExtraPoint) for atom in system.atoms), 23)
+
+        # Check that the parameters match expectation from a known implementation
+        ep1 = system.atoms[7]
+        origin_weights, x_weights, y_weights, local_position = ep1.frame_type.get_weights()
+        self.assertAlmostEqual(local_position[0], -0.11970705016398403)
+        self.assertAlmostEqual(local_position[1], 0.005739964140425085)
+        self.assertAlmostEqual(local_position[2], 0.32884232536689074)
+        self.assertEqual(x_weights, [-1, 0, 1])
+        self.assertEqual(y_weights, [0, -1, 1])
+        self.assertEqual(origin_weights, [1, 0, 0])
+
+        # Now try a virtual site with a negative distance
+        ep2 = system.atoms[22]
+        origin_weights, x_weights, y_weights, local_position = ep2.frame_type.get_weights()
+        self.assertAlmostEqual(local_position[0], -0.34999999466919514)
+        self.assertAlmostEqual(local_position[1], 6.108652257915055e-05)
+        self.assertAlmostEqual(local_position[2], 1.0661609584247726e-08)
+        self.assertEqual(x_weights, [-1, 0.5, 0.5])
+        self.assertEqual(y_weights, [0, -1, 1])
+        self.assertEqual(origin_weights, [1, 0, 0])
+
+        # Check some Drude particle parameters
+        drude1 = system.atoms[2]
+        self.assertIsInstance(drude1, DrudeAtom)
+        self.assertEqual(drude1.alpha, -1.028)
+        self.assertEqual(drude1.thole, 1.3)
+
+        drude2 = system.atoms[-1]
+        self.assertIsInstance(drude2, DrudeAtom)
+        self.assertEqual(drude2.alpha, -0.157)
+        self.assertEqual(drude2.thole, 1.3)
