@@ -1,16 +1,15 @@
 """ Tests some OpenMM-specific functionality """
-from __future__ import print_function, division, absolute_import
-
 from collections import OrderedDict
+from io import StringIO
 import math
 import os
 import unittest
 import warnings
+from unittest import skipIf
 
 import numpy as np
 
 import parmed as pmd
-from parmed.utils.six import StringIO
 from io import TextIOWrapper
 from parmed.charmm import (CharmmPsfFile, CharmmCrdFile, CharmmRstFile,
                            CharmmParameterSet)
@@ -18,10 +17,6 @@ from parmed import openmm, load_file, exceptions, ExtraPoint, unit as u
 from utils import (get_fn, mm, app, has_openmm, has_networkx, has_lxml,
                    FileIOTestCase, CPU, TestCaseRelative, EnergyTestCase)
 from parmed.exceptions import ParameterWarning
-try:
-    from openmm import app
-except ImportError:
-    pass
 
 @unittest.skipUnless(has_openmm, "Cannot test without OpenMM")
 class TestOpenMM(FileIOTestCase, EnergyTestCase):
@@ -29,8 +24,7 @@ class TestOpenMM(FileIOTestCase, EnergyTestCase):
     def setUp(self):
         super(TestOpenMM, self).setUp()
         # Take one of the distributed OpenMM FF XML files as a test
-        self.ffxml = os.path.join(os.path.split(app.__file__)[0], 'data',
-                                  'amber99sbildn.xml')
+        self.ffxml = os.path.join(os.path.split(app.__file__)[0], 'data', 'amber99sbildn.xml')
         super(TestOpenMM, self).setUp()
 
     def test_format_id(self):
