@@ -2,34 +2,25 @@
 Useful functions for the test cases
 """
 import os
-from os.path import join, split, abspath
+from os.path import join
 import random
 import tempfile
 import unittest
-import warnings
 from pathlib import Path
-from typing import Union
 
 import numpy as np
 
 from parmed import gromacs, openmm
-from parmed.utils.six import string_types
-from parmed.utils.six.moves import zip
 
 try:
-    from simtk import openmm as mm
-    from simtk.openmm import app
-    try:
-        openmm_version = tuple([int(x) for x in mm.__version__.split('.')])
-    except AttributeError:
-        import openmm as mm
-        openmm_version = tuple([int(x) for x in mm.__version__.split('.')])
+    import openmm as mm
+    from openmm import app
     CPU = mm.Platform.getPlatformByName('CPU')
     Reference = mm.Platform.getPlatformByName('Reference')
     has_openmm = True
 except ImportError:
     has_openmm = False
-    app = openmm_version = CPU = Reference = mm = None
+    app = CPU = Reference = mm = None
 
 try:
     import networkx as nx
@@ -133,9 +124,6 @@ def get_fn(filename):
     ----------
     filename : str
         Name of the file to get
-    written : bool=False
-        Was this file written by a test? (If so, it is put in a different
-        location)
 
     Returns
     -------
@@ -201,7 +189,7 @@ def diff_files(file1, file2, ignore_whitespace=True,
         raise ValueError('Cannot specify absolute_error AND relative_error')
     if absolute_error is not None: absolute_error = float(absolute_error)
     if relative_error is not None: relative_error = float(relative_error)
-    if isinstance(file1, string_types):
+    if isinstance(file1, str):
         try:
             f1 = open(file1, 'r')
         except IOError:
@@ -210,7 +198,7 @@ def diff_files(file1, file2, ignore_whitespace=True,
     else:
         f1 = file1
         file1 = str(file1)
-    if isinstance(file2, string_types):
+    if isinstance(file2, str):
         try:
             f2 = open(file2, 'r')
         except IOError:
@@ -558,8 +546,8 @@ def equal_atoms(tester, a1, a2):
         else:
             tester.assertFalse(hasattr(a1, key))
 
-def is_jenkins():
-    return 'JENKINS_URL' in os.environ
+def is_local():
+    return 'CI' not in os.environ
 
 def has_old_vec3():
     from parmed.vec3 import Vec3
